@@ -14,7 +14,7 @@
  * The Original Code is Enigmail.
  *
  * The Initial Developer of the Original Code is Patrick Brunschwig.
- * Portions created by Patrick Brunschwig <patrick@mozilla-enigmail.org> are
+ * Portions created by Patrick Brunschwig <patrick@enigmail.net> are
  * Copyright (C) 2005 Patrick Brunschwig. All Rights Reserved.
  *
  * Contributor(s):
@@ -33,32 +33,38 @@
  * ***** END LICENSE BLOCK ***** */
 
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
+Components.utils.import("resource://enigmail/enigmailCore.jsm");
 Components.utils.import("resource://enigmail/keyManagement.jsm");
+Components.utils.import("resource://enigmail/installGnuPG.jsm");
+Components.utils.import("resource://enigmail/commonFuncs.jsm");
+Components.utils.import("resource://enigmail/passwordCheck.jsm");
 
 // const Ec is already defined in enigmailKeygen.js
 
-var gEnigModifySettings;
-var gLastDirection=0;
+var gLastDirection = 0;
+var gWizardUserMode = "beginner";
 var gEnigAccountMgr;
 var gPubkeyFile = {value: null};
 var gSeckeyFile = {value: null};
 var gCreateNewKey=false;
 var gPrefEnigmail;
-
-
-EnigInitCommon("enigmailSetupWizard");
+var gDownoadObj = null;
+var gPassPhraseQuality = null;
+var gPageStack = [];  // required for correct stepping back
 
 function onLoad() {
-  gEnigModifySettings = {
-    imapOnDemand: true,
-    flowedText: true,
-    viewPlainText: true,
-    quotedPrintable: true,
-    composeHTML: true
-  };
   gEnigAccountMgr = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
 
   fillIdentities('checkbox');
+
+  let winOptions = EnigGetWindowOptions();
+  if ("skipIntro" in winOptions) {
+    if (winOptions["skipIntro"] == "true") {
+      var wizard = getWizard();
+      wizard.goTo("pgWelcome");
+
+    }
+  }
 }
 
 
@@ -73,55 +79,242 @@ function onCancel() {
     }
   }
   else {
-    return (EnigLongAlert(EnigGetString("setupWizard.reallyCancel"), null, EnigGetString("dlg.button.close"), EnigGetString("dlg.button.continue")) == 0);
+    var r=(EnigLongAlert(EnigGetString("setupWizard.reallyCancel"), null, EnigGetString("dlg.button.close"), EnigGetString("dlg.button.continue")) == 0);
+
+    if (r && gDownoadObj) {
+      gDownoadObj.abort();
+      gDownoadObj = null;
+    }
+
+    return r;
   }
 }
 
 
+function getWizard() {
+  return document.getElementById("enigmailSetupWizard");
+}
+
 function setLastPage() {
-  var wizard = document.getElementById("enigmailSetupWizard");
+  var wizard = getWizard();
+
   if (wizard.currentPage) {
-    wizard.setAttribute("lastViewedPage", wizard.currentPage.pageid);
+    gPageStack.push(wizard.currentPage.id);
   }
 }
 
 function onBack() {
   DEBUG_LOG("onBack");
+  var wizard = getWizard();
   gLastDirection=-1;
-  setLastPage();
+
+  gPageStack.pop();
 }
 
+function onPageShow() {
+  var wizard = getWizard();
+  wizard.canRewind = (gPageStack.length > 0)
+}
+
+/***
+ * State machine to decide which is the next page to show in the wizard.
+ * This function is called when the "Next" button is pressed, before the
+ * next page is opened, allowing to still modify the wiazrd's sequence.
+ */
+
 function onNext() {
-  DEBUG_LOG("onNext");
-  gLastDirection=1;
-  setLastPage();
-  var wizard = document.getElementById("enigmailSetupWizard");
-  if (wizard.currentPage) {
-    switch(wizard.currentPage.pageid) {
-    case "pgKeySel":
-      wizardSelKey();
-      break;
-    case "pgWelcome":
-      checkIdentities();
-      break;
-    case "pgSettings":
-      return loadKeys();
-    case "pgKeyCreate":
-      return checkPassphrase();
+
+  var enableNext = true;
+
+  // private function to se the next page of the wizard
+  function setNextPage(pageId) {
+    if (pageId == "") {
+      enableNext = false;
+    }
+    else {
+      let wizard = getWizard();
+      wizard.currentPage.next = pageId;
+      enableNext = true;
     }
   }
 
-  return true;
+  DEBUG_LOG("onNext\n");
+  gLastDirection=1;
+  setLastPage();
+  var wizard = getWizard();
+  if (wizard.currentPage) {
+    switch(wizard.currentPage.pageid) {
+    case "pgWelcome":
+      setNextPage(onAfterPgWelcome());
+      break;
+    case "pgInstallGnuPG":
+      setNextPage(onAfterPgInstallGnuPG());
+      break;
+    case "pgSelectId":
+      setNextPage(onAfterPgSelectId());
+      break;
+    case "pgKeySel":
+      setNextPage(onAfterPgKeySel());
+      break;
+    case "pgNoKeyFound":
+      setNextPage(onAfterPgNoKeyFound());
+      break;
+    case "pgKeyImport":
+      setNextPage(onAfterPgKeyImport());
+      break;
+    case "pgKeyCreate":
+      setNextPage(onAfterPgKeyCreate());
+      break;
+    case "pgKeygen":
+      setNextPage(onAfterPgKeygen());
+      break;
+    }
+  }
+  return enableNext;
 }
 
-function setNextPage(pageId) {
-  var wizard = document.getElementById("enigmailSetupWizard");
-  wizard.currentPage.next = pageId;
+/**** State machine helper functions ****
+ *
+ * All functions:
+ * return the next page ID or "" in case the 'Next' button should be disabled
+ */
+function onAfterPgWelcome() {
+
+  if (checkGnupgInstallation()) {
+    let hasSecretKeys = checkSecretKeys();
+    switch (gWizardUserMode) {
+    case "beginner":
+      if (hasSecretKeys) {
+        loadKeys();
+        return "pgKeySel";
+      }
+      else {
+        return "pgKeyCreate";
+      }
+    case "advanced":
+      if (countIdentities() > 1) {
+        return "pgSelectId";
+      }
+      else {
+        if (hasSecretKeys) {
+          loadKeys();
+          return "pgKeySel";
+        }
+        else {
+          return "pgNoKeyFound";
+        }
+      }
+    case "expert":
+      return "pgExpert";
+    }
+  }
+  else {
+    return "pgInstallGnuPG";
+  }
 }
 
+function onAfterPgInstallGnuPG() {
+  let hasSecretKeys = checkSecretKeys();
+  switch (gWizardUserMode) {
+  case "beginner":
+    if (hasSecretKeys) {
+      loadKeys();
+      return "pgKeySel";
+    }
+    else {
+      return "pgKeyCreate";
+    }
+  case "advanced":
+    if (countIdentities() > 1) {
+      return "pgSelectId";
+    }
+    else {
+      if (hasSecretKeys) {
+        loadKeys();
+        return "pgKeySel";
+      }
+      else {
+        return "pgNoKeyFound";
+      }
+    }
+  case "expert":
+    return "pgExpert";
+  }
+}
+
+function onAfterPgSelectId() {
+  let hasSecretKeys = checkSecretKeys();
+  if (hasSecretKeys) {
+    loadKeys();
+    return "pgKeySel";
+  }
+  else {
+    return "pgNoKeyFound";
+  }
+}
+
+function onAfterPgKeySel() {
+  if (gCreateNewKey) {
+    return "pgKeyCreate";
+  }
+  else {
+    return "pgComplete";
+  }
+}
+
+function onAfterPgNoKeyFound() {
+  if (gCreateNewKey) {
+    return "pgKeyCreate";
+  }
+  else {
+    return "pgKeyImport";
+  }
+}
+
+function onAfterPgKeyImport() {
+  return "pgKeySel";
+}
+
+function onAfterPgKeyCreate() {
+  if (checkPassphrase()) {
+    return "pgKeygen";
+  }
+
+  return "";
+}
+
+function onAfterPgKeygen() {
+  return "pgComplete";
+}
+
+/**
+ * Check if GnuPG is available
+ */
+function checkGnupgInstallation() {
+  var s = enigGetSvc(true);
+  return (s ? true : false);
+}
+
+/**
+ * Check if secret keys are available
+ */
+function checkSecretKeys() {
+  var keyList = Ec.getSecretKeys(null, true);
+  if (keyList && keyList.length > 0) {
+    return true;
+  }
+
+  return false;
+}
+
+/**
+ * Enable or disable the "Next" (or "Done") button
+ *
+ * disable: boolean: true = button is disabled / false = button is enabled
+ */
 function disableNext(disable) {
-  var wizard = document.getElementById("enigmailSetupWizard");
-  wizard.getButton("next").disabled = disable;
+  var wizard = getWizard();
+  wizard.canAdvance = !disable;
 }
 
 
@@ -137,6 +330,116 @@ function countSelectedId() {
   return idCount;
 }
 
+function onShowPgInstallGnuPG() {
+  var ok = enigGetSvc(true);
+  disableNext(!ok);
+
+  if (InstallGnuPG.checkAvailability()) {
+    document.getElementById("installBox").removeAttribute("collapsed");
+  }
+  else {
+    document.getElementById("findGpgBox").removeAttribute("collapsed");
+  }
+}
+
+
+
+function installGnuPG() {
+  var progressBox = document.getElementById("progressBox");
+  var downloadProgress = document.getElementById("downloadProgress");
+  var installLabel = document.getElementById("installLabel");
+  var installProgress = document.getElementById("installProgress");
+  var btnInstallGnupg = document.getElementById("btnInstallGnupg");
+  var btnLocateGnuPG = document.getElementById("btnLocateGnuPG");
+
+  btnInstallGnupg.setAttribute("disabled", true);
+  btnLocateGnuPG.setAttribute("disabled", true);
+  progressBox.removeAttribute("collapsed");
+
+  InstallGnuPG.startInstaller({
+    onStart: function(reqObj) {
+      gDownoadObj = reqObj;
+    },
+
+    onError: function (errorMessage) {
+      if (typeof(errorMessage) == "object") {
+        var s = EnigGetString("errorType."+errorMessage.type);
+        if (errorMessage.type.startsWith("Security")) {
+          s += "\n"+ EnigGetString("setupWizard.downloadForbidden");
+        }
+        else
+          s += "\n"+ EnigGetString("setupWizard.downloadImpossible");
+
+        EnigAlert(s);
+      }
+      else {
+        EnigAlert(EnigGetString(errorMessage));
+      }
+
+      this.returnToDownload();
+    },
+
+    onWarning: function(message) {
+      var ret = false;
+      if (message == "hashSumMismatch") {
+        ret = EnigConfirm(EnigGetString("setupWizard.hashSumError"), EnigGetString("dlgYes"),
+                  EnigGetString("dlgNo"));
+      }
+
+      if (! ret) this.returnToDownload();
+
+      return ret;
+    },
+
+    onProgress: function(event) {
+      if (event.lengthComputable) {
+        var percentComplete = event.loaded / event.total * 100;
+        downloadProgress.setAttribute("value", percentComplete);
+      }
+      else {
+        downloadProgress.setAttribute("mode", "undetermined");
+      }
+    },
+
+    onDownloaded: function() {
+      gDownoadObj = null;
+      downloadProgress.setAttribute("value", 100);
+      installLabel.removeAttribute("collapsed");
+      installProgress.removeAttribute("collapsed");
+    },
+
+
+    returnToDownload: function() {
+      btnInstallGnupg.removeAttribute("disabled");
+      btnLocateGnuPG.removeAttribute("disabled");
+      progressBox.setAttribute("collapsed", "true");
+      downloadProgress.setAttribute("value", 0);
+      installLabel.setAttribute("collapsed", "true");
+      installProgress.setAttribute("collapsed", "true");
+    },
+
+    onLoaded: function() {
+      installProgress.setAttribute("value", 100);
+      installProgress.setAttribute("mode", "determined");
+
+      document.getElementById("installComplete").removeAttribute("collapsed");
+
+      var origPath = EnigGetPref("agentPath");
+      EnigSetPref("agentPath", "");
+
+      var s = enigGetSvc(true);
+      if (s) {
+        disableNext(false);
+      }
+      else {
+        EnigSetPref("agentPath", origPath);
+        this.returnToDownload();
+        EnigAlert(EnigGetString("setupWizard.installFailed"));
+      }
+    }
+  });
+}
+
 function browseKeyFile(referencedId, referencedVar) {
   var filePath = EnigFilePicker(EnigGetString("importKeyFile"),
                                "", false, "*.asc", "",
@@ -149,27 +452,31 @@ function browseKeyFile(referencedId, referencedVar) {
 }
 
 function importKeyFiles() {
+  Ec.DEBUG_LOG("enigmailSetupWizard.js: importKeyFiles\n");
   if (document.getElementById("publicKeysFile").value.length == 0) {
-    EnigAlert(EnigString("setupWizard.specifyFile"));
+    EnigAlert(EnigGetString("setupWizard.specifyFile"));
     return false;
   }
 
   var importedKeys;
   var exitCode;
 
-  var enigmailSvc = enigGetSvc();
-  if (! enigmailSvc) return;
+  var enigmailSvc = enigGetSvc(false);
+  if (! enigmailSvc) return false;
+
+  disableNext(true);
 
   var errorMsgObj = {};
   var keyListObj = {};
-  var exitCode = enigmailSvc.importKeyFromFile(window, gPubkeyFile.value, errorMsgObj, keyListObj);
+  exitCode = enigmailSvc.importKeyFromFile(window, gPubkeyFile.value, errorMsgObj, keyListObj);
   if (exitCode != 0) {
     EnigAlert(EnigGetString("importKeysFailed")+"\n\n"+errorMsgObj.value);
     return false;
   }
   importedKeys = keyListObj.value;
 
-  if (document.getElementById("privateKeysFile").value.length > 0) {
+  if (document.getElementById("privateKeysFile").value.trim().length > 0) {
+    Ec.DEBUG_LOG("enigmailSetupWizard.js: importKeyFiles - private Keys\n");
 
     exitCode = enigmailSvc.importKeyFromFile(window, gSeckeyFile.value, errorMsgObj, keyListObj);
     if (exitCode != 0) {
@@ -180,8 +487,11 @@ function importKeyFiles() {
   }
 
 
-  var exitCode = 0;
+  exitCode = 0;
   var keyList=importedKeys.split(/;/);
+
+  Ec.DEBUG_LOG("enigmailSetupWizard.js: importKeyFiles - importing "+ keyList.length +" keys\n");
+
   setKeyTrustNextKey(keyList, 0);
 
   return true;
@@ -190,8 +500,24 @@ function importKeyFiles() {
 function setKeyTrustNextKey(keyList, index) {
   Ec.DEBUG_LOG("enigmailSetupWizard.js: setKeyTrustNextKey("+index+")\n");
 
+  if (index == keyList.length) {
+    // end of list reached
+    Ec.recalcTrustDb();
+    loadKeys();
+    return;
+  }
+
   var aKey = keyList[index].split(/:/);
-  if (Number(aKey[1]) & 16) {
+
+  let keyType;
+  try {
+    keyType = Number(aKey[1]);
+  }
+  catch (ex) {
+    keyType = 0;
+  }
+
+  if (keyType & 16) {
     // imported key contains secret key
     EnigmailKeyMgmt.setKeyTrust(window, aKey[0], 5,
       function(exitCode, errorMsg) {
@@ -199,22 +525,16 @@ function setKeyTrustNextKey(keyList, index) {
           return;
         }
 
-        ++index;
         if (index < keyList.length) {
-          setKeyTrustNextKey(keyList, index);
+          setKeyTrustNextKey(keyList, index + 1);
         }
-        else
-          loadKeys();
       }
     );
   }
   else {
-    ++index;
     if (index < keyList.length) {
-      setKeyTrustNextKey(keyList, index);
+      setKeyTrustNextKey(keyList, index + 1);
     }
-    else
-      loadKeys();
   }
 }
 
@@ -224,6 +544,18 @@ function displayKeyCreate() {
     fillIdentities('menulist');
   }
 
+  gPassPhraseQuality = document.getElementById("passphraseQuality");
+
+  // gpg 2.1.0 and 2.1.1 queries passphrase only gpg-agent only
+  if (! Ec.getGpgFeature("keygen-passphrase")) {
+    document.getElementById("keyCreateDescSec1").setAttribute("collapsed", "true");
+    document.getElementById("passphraseBox").setAttribute("collapsed", "true");
+    document.getElementById("keyCreateDescSec2").removeAttribute("collapsed");
+  }
+  else {
+    checkPassphrasesEqual();
+  }
+
   if (countSelectedId() == 1) {
     var node = document.getElementById("idSelection").firstChild;
     while (node) {
@@ -231,10 +563,12 @@ function displayKeyCreate() {
         var identity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
         var idName = identity.identityName;
 
-        var serverSupports = getServersForIdentity(gEnigAccountMgr, identity);
+        var serverList = queryISupArray(
+                getServersForIdentity(gEnigAccountMgr, identity),
+                Components.interfaces.nsIMsgIncomingServer);
 
-        if (serverSupports.GetElementAt(0)) {
-          var inServer = serverSupports.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
+        if (serverList.length > 0) {
+          var inServer = serverList[0];
 
           idName += " - "+inServer.prettyName;
         }
@@ -251,6 +585,34 @@ function displayKeyCreate() {
     document.getElementById("userIdentityLabel").setAttribute("collapsed", "true");
     document.getElementById("userIdentity").removeAttribute("collapsed");
   }
+
+}
+
+
+function checkPassphraseQuality(txtBox) {
+  var qualityRes = passwordCheck.checkQuality(txtBox.value);
+
+  if (qualityRes.valid) {
+    gPassPhraseQuality.value = qualityRes.complexity;
+  }
+  else if (txtBox.value.length > 0) {
+    gPassPhraseQuality.value = (qualityRes.complexity / 2);
+  }
+  else {
+    gPassPhraseQuality.value = 0;
+  }
+
+  checkPassphrasesEqual();
+}
+
+/**
+ * Check if entered passphrases are equal. If yes, enable the next button
+ */
+function checkPassphrasesEqual() {
+  let p1 = document.getElementById("passphrase").value;
+  let p2 = document.getElementById("passphraseRepeat").value;
+
+  disableNext(p1.length == 0 || p1 != p2);
 }
 
 
@@ -264,8 +626,9 @@ function displayKeySel() {
   }
 }
 
+function clearKeyListEntries() {
+  Ec.DEBUG_LOG("enigmailSetupWizard.js: clearKeyListEntries\n");
 
-function clearKeyListEntries(){
   // remove all rows
   var treeChildren = document.getElementById("uidSelectionChildren");
   while (treeChildren.firstChild) {
@@ -273,55 +636,72 @@ function clearKeyListEntries(){
   }
 }
 
-function onSetStartNow(doStart) {
-  if (doStart) {
-    setNextPage("pgSelectId");
+function onSetStartNow(mode) {
+  var wizard = getWizard();
+  if (mode == 0) {
+    wizard.lastPage = true;
+    wizard.getButton("next").hidden = true;
+    wizard.getButton("finish").hidden = false;
+    wizard.currentPage.next = "";
   }
   else {
-    setNextPage("pgNoStart")
+    wizard.lastPage = false;
+    wizard.getButton("next").hidden = false;
+    wizard.getButton("finish").hidden = true;
+    wizard.currentPage.next = "pgWelcome";
   }
 }
 
+function onSetSelectMode(userMode) {
+  gWizardUserMode = userMode;
+}
+
 function onKeySelected() {
-  var wizard = document.getElementById("enigmailSetupWizard");
+  var wizard = getWizard();
   var uidSel = document.getElementById("uidSelection");
+
+  if (uidSel.view.selection.count == 1) {
+    var currIndex = uidSel.view.selection.currentIndex;
+    var currItem = uidSel.view.getItemAtIndex(currIndex);
+    gGeneratedKey = currItem.getAttribute("keyId");
+  }
+  else {
+    gGeneratedKey = null;
+  }
+
   disableNext(uidSel.view.selection.count == 0);
 }
 
 function wizardSetFocus() {
-  document.getElementById("startNow").focus();
+  document.getElementById("selectMode").focus();
+  disableNext(false);
+
 }
 
 function loadKeys() {
-  var wizard = document.getElementById("enigmailSetupWizard");
+  Ec.DEBUG_LOG("enigmailSetupWizard.js: loadKeys\n");
 
-  var enigmailSvc = enigGetSvc();
+  var enigmailSvc = enigGetSvc(false);
 
   if (!enigmailSvc) {
-    return false;
+    return;
   }
   clearKeyListEntries();
 
   var exitCodeObj = {};
   var statusFlagsObj = {};
   var errorMsgObj = {};
-  var keyList = Ec.getSecretKeys(window);
+  var keyList = Ec.getSecretKeys(window, true);
   if (keyList == null) {
-    return false;
+    return;
   }
 
-
-  if (keyList.length ==0) {
-    setNextPage("pgNoKeyFound");
-    return true;
-  }
-
-  var uidChildren = document.getElementById("uidSelectionChildren")
+  var uidChildren = document.getElementById("uidSelectionChildren");
   for (i=0; i < keyList.length; i++) {
     var item = uidChildren.appendChild( document.createElement('treeitem') );
     item.setAttribute("keyId", keyList[i].id);
     var row = item.appendChild(document.createElement('treerow'));
-    var cell = row.appendChild( document.createElement('treecell') )
+    var cell = row.appendChild( document.createElement('treecell') );
     cell.setAttribute('label', keyList[i].name);
     cell.setAttribute('observes', "bcKeyEnabled");
     cell = row.appendChild( document.createElement('treecell') );
@@ -335,10 +715,12 @@ function loadKeys() {
   return true;
 }
 
-function enigGetSvc() {
+function enigGetSvc(resetCheck) {
   // Lazy initialization of enigmail JS component (for efficiency)
   // variant of GetEnigmailSvc function
 
+  if (resetCheck) gEnigmailSvc = null;
+
   if (gEnigmailSvc) {
     return gEnigmailSvc.initialized ? gEnigmailSvc : null;
   }
@@ -353,12 +735,12 @@ function enigGetSvc() {
 
   DEBUG_LOG("enigmailWizard.js: gEnigmailSvc = "+gEnigmailSvc+"\n");
 
-  while (!gEnigmailSvc.initialized) {
+  if (!gEnigmailSvc.initialized) {
     // Try to initialize enigmail
 
     if (! gPrefEnigmail) {
-      EnigmailCommon.initPrefService();
-      gPrefEnigmail = EnigmailCommon.prefBranch;
+      EnigmailCore.initPrefService();
+      gPrefEnigmail = EnigmailCore.prefBranch;
     }
 
     try {
@@ -372,19 +754,8 @@ function enigGetSvc() {
 
     } catch (ex) {
 
-      // Display initialization error alert
-      EnigAlert(EnigGetString("setupWizard.locateGpg"));
-      var gpgPath = wizardLocateGpg();
-      if (! gpgPath) {
-        if (onCancel()) {
-          window.close();
-          return null;
-        }
-      }
-      else {
-        EnigSetPref("agentPath", EnigGetFilePath(gpgPath));
-      }
-    }
+      return null;
+     }
 
     var configuredVersion = EnigGetPref("configuredVersion");
 
@@ -395,35 +766,64 @@ function enigGetSvc() {
   return gEnigmailSvc.initialized ? gEnigmailSvc : null;
 }
 
+
+
 function wizardLocateGpg() {
-  var fileName="gpg";
-  var ext="";
-  if (navigator.platform.search(/Win/i) == 0) {
-    ext=".exe";
+  var fileName = "gpg";
+  var ext = "";
+  if (Ec.isDosLike()) {
+    ext = ".exe";
   }
   var filePath = EnigFilePicker(EnigGetString("locateGpg"),
                            "", false, ext,
                            fileName+ext, null);
-  return filePath;
+
+  if (filePath) {
+    EnigSetPref("agentPath", EnigGetFilePath(filePath));
+    var svc = enigGetSvc(true);
+
+    if (! svc) {
+      EnigAlert(EnigGetString("setupWizard.invalidGpg"));
+    }
+    else {
+      document.getElementById("gpgFoundBox").removeAttribute("collapsed");
+      disableNext(false);
+    }
+  }
+
 }
 
 function checkPassphrase() {
 
-  var passphrase = enigmailCheckPassphrase();
-  if (passphrase == null) return false;
+  // gpg >= 2.1 queries passphrase using gpg-agent only
 
-  if (passphrase.length < 8) {
-    EnigAlert(EnigGetString("passphrase.min8keys"));
-    return false;
+  if (Ec.getGpgFeature("keygen-passphrase")) {
+    var passphrase = enigmailCheckPassphrase();
+    if (passphrase == null) return false;
+
+    if (passphrase.length < 8) {
+      EnigAlert(EnigGetString("passphrase.min8keys"));
+      return false;
+    }
   }
+
   return true;
 
 }
 
 function wizardGenKey() {
-  var wizard = document.getElementById("enigmailSetupWizard");
+  var wizard = getWizard();
+
+  disableNext(true);
+  wizard.canRewind = false;
+
   var passphrase = document.getElementById("passphrase").value;
 
+  // gpg >= 2.1 queries passphrase using gpg-agent only
+  if (! Ec.getGpgFeature("keygen-passphrase")) {
+    passphrase = "";
+  }
+
   var curId = wizardGetSelectedIdentity();
 
   var userName = curId.fullName;
@@ -445,16 +845,14 @@ function wizardGenKey() {
           gGeneratedKey = gGeneratedKey.replace(/(.*\[GNUPG:\] KEY_CREATED . )([a-fA-F0-9]+)([\n\r].*)*/, "$2");
           gAllData = gAllData.replace(/\[GNUPG:\] KEY_CREATED . [a-fA-F0-9]+[\n\r]/, "");
         }
-        gAllData = gAllData.replace(/[\r\n]*\[GNUPG:\] GOOD_PASSPHRASE/g, "").replace(/([\r\n]*\[GNUPG:\] PROGRESS primegen )(.)( \d+ \d+)/g, "$2")
+        gAllData = gAllData.replace(/[\r\n]*\[GNUPG:\] GOOD_PASSPHRASE/g, "").replace(/([\r\n]*\[GNUPG:\] PROGRESS primegen )(.)( \d+ \d+)/g, "$2");
         var progMeter = document.getElementById("keygenProgress");
         var progValue = Number(progMeter.value);
         progValue += (1+(100-progValue)/200);
         if (progValue >= 95) progValue=10;
         progMeter.setAttribute("value", progValue);
       }
-  }
-  wizard.getButton("next").disabled = true
-  wizard.getButton("back").disabled = true;
+  };
 
   try {
     gKeygenRequest = Ec.generateKey(window,
@@ -462,7 +860,7 @@ function wizardGenKey() {
                        "",
                        userEmail,
                        365*5 /* 5 years */,
-                       2048,
+                       4096,
                        ENIG_KEYTYPE_RSA,
                        passphrase,
                        listener);
@@ -480,52 +878,71 @@ function wizardGenKey() {
   return false;
 }
 
-function wizardSelKey() {
-  // use existing key
-  var createKey=document.getElementById("createPgpKey");
-  if (createKey.value == "0") {
-    var uidSel = document.getElementById("uidSelection");
-    var currIndex = uidSel.view.selection.currentIndex;
-    var currItem = uidSel.view.getItemAtIndex(currIndex);
-    gGeneratedKey = currItem.getAttribute("keyId");
-  }
+function wizardKeygenTerminate(exitCode)
+{
+  DEBUG_LOG("enigmailSetupWizard.js: wizardKeygenTerminate\n");
+
+  // Give focus to this window
+  window.focus();
+
+  gKeygenRequest = null;
+
+  if ((! gGeneratedKey) || gGeneratedKey == KEYGEN_CANCELLED) return;
+
+  var progMeter = document.getElementById("keygenProgress");
+  progMeter.setAttribute("value", 100);
+
+
+  enigmailKeygenCloseRequest();
+  var enigmailSvc = enigGetSvc(false);
+  enigmailSvc.invalidateUserIdList();
+
+  document.getElementById("revCertBox").removeAttribute("hidden");
 }
 
+// create a revokation certificate
 
-function queryISupportsArray(supportsArray, iid) {
-    var result = new Array;
-    for (var i=0; i<supportsArray.Count(); i++) {
-      result[i] = supportsArray.GetElementAt(i).QueryInterface(iid);
+function wizardCreateRevCert() {
+  DEBUG_LOG("enigmailSetupWizard.js: wizardCreateRevCert\n");
+
+  let curId = wizardGetSelectedIdentity();
+
+  genAndSaveRevCert(gGeneratedKey, curId.email).then(
+    function _resolve() {
+      disableNext(false);
+    },
+    function _reject() {
+      disableNext(true);
     }
-    return result;
+  );
+}
+
+function queryISupArray(supportsArray, iid) {
+  var result = [];
+  var i;
+
+  for (i=0; i<supportsArray.length; i++) {
+    result.push(supportsArray.queryElementAt(i, iid));
+  }
+
+  return result;
 }
 
 function countIdentities() {
   var accountManager = Components.classes[ENIG_ACCOUNT_MANAGER_CONTRACTID].getService(Components.interfaces.nsIMsgAccountManager);
   var idSupports = accountManager.allIdentities;
-  var identities = queryISupportsArray(idSupports,
+  var identities = queryISupArray(idSupports,
                                        Components.interfaces.nsIMsgIdentity);
   return identities.length;
 }
 
-function checkIdentities() {
-  var wizard = document.getElementById("enigmailSetupWizard");
-
-  if (wizard.currentPage.next != "pgNoStart") {
-    if (countIdentities() <= 1) {
-      setNextPage("pgSign");
-    }
-  }
-}
-
-
 function fillIdentities(fillType)
 {
   DEBUG_LOG("enigmailSetupWizard.js: fillIdentities\n");
 
   var defIdentity;
   var parentElement;
-  var identities = queryISupportsArray(gEnigAccountMgr.allIdentities,
+  var identities = queryISupArray(gEnigAccountMgr.allIdentities,
                                        Components.interfaces.nsIMsgIdentity);
 
   if (fillType == "checkbox") {
@@ -536,21 +953,10 @@ function fillIdentities(fillType)
 
     // Find out default identity
     var defIdentities = gEnigAccountMgr.defaultAccount.identities;
-    try {
-      // Gecko >= 20
-      if (defIdentities.length >= 1) {
-        defIdentity = defIdentities.queryElementAt(0, Components.interfaces.nsIMsgIdentity);
-      } else {
-        defIdentity = identities[0];
-      }
-    }
-    catch (ex) {
-      // Gecko < 20
-      if (defIdentities.Count() >= 1) {
-        defIdentity = defIdentities.QueryElementAt(0, Components.interfaces.nsIMsgIdentity);
-      } else {
-        defIdentity = identities[0];
-      }
+    if (defIdentities.length >= 1) {
+      defIdentity = defIdentities.queryElementAt(0, Components.interfaces.nsIMsgIdentity);
+    } else {
+      defIdentity = identities[0];
     }
 
     if (document.getElementById("activateId").value == "0") {
@@ -597,10 +1003,12 @@ function fillIdentities(fillType)
     if (!identity.valid || !identity.email)
       continue;
 
-    var serverSupports = getServersForIdentity(gEnigAccountMgr, identity);
+    var serverList = queryISupArray(
+            getServersForIdentity(gEnigAccountMgr, identity),
+            Components.interfaces.nsIMsgIncomingServer);
 
-    if (serverSupports.GetElementAt(0)) {
-      var inServer = serverSupports.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
+    if (serverList.length > 0) {
+      var inServer = serverList[0];
 
       var accountName = " - "+inServer.prettyName;
 
@@ -628,7 +1036,7 @@ function fillIdentities(fillType)
 
       if (fillType != "checkbox") {
         // pre-select default ID
-        var idList = document.getElementById("userIdentity")
+        var idList = document.getElementById("userIdentity");
         if (!selected)
           idList.selectedItem = item;
         if (identity.key == defIdentity.key) {
@@ -650,18 +1058,23 @@ function wizardGetSelectedIdentity()
 
 function applyWizardSettings() {
   DEBUG_LOG("enigmailSetupWizard.js: applyWizardSettings\n");
+  debugger;
 
   loadLastPage();
 
+  Ec.setPref("encryptionModel", 0);
+
   if (document.getElementById("activateId").value == "1") {
+    // activate all identities
     var idSupports = gEnigAccountMgr.allIdentities;
-    var identities = queryISupportsArray(idSupports,
+    var identities = queryISupArray(idSupports,
                                        Components.interfaces.nsIMsgIdentity);
     for (var i=0; i<identities.length; i++) {
       wizardApplyId(identities[i], gGeneratedKey);
     }
   }
   else {
+    // activate selected identities
     var node = document.getElementById("idSelection").firstChild;
     while (node) {
       if (node.checked) {
@@ -672,39 +1085,38 @@ function applyWizardSettings() {
     }
   }
 
-  applyMozSetting("imapOnDemand", "mail.server.default.mime_parts_on_demand", false);
-  applyMozSetting("flowedText" ,"mailnews.send_plaintext_flowed", false)
-  applyMozSetting("quotedPrintable", "mail.strictly_mime", false);
-  applyMozSetting("viewPlainText", "mailnews.display.html_as", 1);
-  applyMozSetting("viewPlainText", "mailnews.display.prefer_plaintext", true);
+  applyMozSetting("mail.server.default.mime_parts_on_demand", false);
+  applyMozSetting("mailnews.send_plaintext_flowed", false);
+  applyMozSetting("mail.strictly_mime", false);
 
   EnigSetPref("configuredVersion", EnigGetVersion());
   EnigSavePrefs();
 }
 
-function applyMozSetting(param, preference, newVal) {
-  if (gEnigModifySettings[param]) {
-    if (typeof(newVal)=="boolean") {
-      EnigmailCommon.prefRoot.setBoolPref(preference, newVal);
-    }
-    else if (typeof(newVal)=="number") {
-      EnigmailCommon.prefRoot.setIntPref(preference, newVal);
-    }
-    else if (typeof(newVal)=="string") {
-      EnigmailCommon.prefRoot.setCharPref(preference, newVal);
-    }
+function applyMozSetting(preference, newVal)
+{
+  if (typeof(newVal)=="boolean") {
+    EnigmailCore.prefRoot.setBoolPref(preference, newVal);
+  }
+  else if (typeof(newVal)=="number") {
+    EnigmailCore.prefRoot.setIntPref(preference, newVal);
+  }
+  else if (typeof(newVal)=="string") {
+    EnigmailCore.prefRoot.setCharPref(preference, newVal);
   }
 }
 
-function wizardApplyId(identity, keyId) {
+
+function wizardApplyId(identity, keyId)
+{
   DEBUG_LOG("enigmailSetupWizard.js: wizardApplyId: identity.Key="+identity.key+"\n");
   var accountManager = Components.classes[ENIG_ACCOUNT_MANAGER_CONTRACTID].getService(Components.interfaces.nsIMsgAccountManager);
   var idServers = getServersForIdentity(accountManager, identity);
-  var servers = queryISupportsArray(idServers ,Components.interfaces.nsIMsgIncomingServer);
+  var servers = queryISupArray(idServers ,Components.interfaces.nsIMsgIncomingServer);
 
   var newsServer = false;
   for (var i=0; i<servers.length; i++) {
-    newsServer = (servers[i].localStoreType == "news")
+    newsServer = (servers[i].localStoreType == "news");
   }
 
   identity.setBoolAttribute("enablePgp", true);
@@ -712,53 +1124,23 @@ function wizardApplyId(identity, keyId) {
   identity.setCharAttribute("pgpkeyId", "0x"+keyId.substr(-8,8));
   identity.setIntAttribute("openPgpHeaderMode", 0);
 
-  var signMsg = (document.getElementById("signMsg").value== "1");
-  var encryptMsg = ((!newsServer) && (document.getElementById("encryptMsg").value== "1"));
+  // process signing settings:
+  // NOTE: option defaultSigningPolicy is an INT
+  identity.setIntAttribute("defaultSigningPolicy", 0);
+  identity.setBoolAttribute("pgpSignEncrypted", false);
+  identity.setBoolAttribute("pgpSignPlain", false);
 
-  identity.setBoolAttribute("pgpSignEncrypted", signMsg);
-  identity.setBoolAttribute("pgpSignPlain", signMsg);
-  identity.setIntAttribute("defaultEncryptionPolicy", (encryptMsg ? 1 : 0));
-  if ((document.getElementById("changeSettings").value == "1") &&
-      gEnigModifySettings["composeHTML"]) {
-    identity.setBoolAttribute("compose_html", false);
-  }
+  // process encryption settings:
+  //  0: convenient encryption (global preference)
+  //  1: by default encrypt (account preference)
+  //  2: by default don't encrypt (account preference)
+  // NOTE: option defaultEncryptionPolicy is an INT
+  identity.setIntAttribute("defaultEncryptionPolicy", 0);
+
+  // change default composition mode to plain text
+  identity.setBoolAttribute("compose_html", false);
 }
 
-
-function wizardKeygenTerminate(exitCode) {
-  DEBUG_LOG("enigmailSetupWizard.js: wizardKeygenTerminate\n");
-
-  // Give focus to this window
-  window.focus();
-
-  gKeygenRequest = null;
-
-  if ((! gGeneratedKey) || gGeneratedKey == KEYGEN_CANCELLED) return;
-
-  var progMeter = document.getElementById("keygenProgress");
-  progMeter.setAttribute("value", 100);
-
-  var curId = wizardGetSelectedIdentity();
-
-  if (EnigConfirm(EnigGetString("keygenComplete", curId.email)+"\n\n"+EnigGetString("revokeCertRecommended"), EnigGetString("keyMan.button.generateCert"), EnigGetString("dlg.button.skip"))) {
-    EnigCreateRevokeCert(gGeneratedKey, curId.email, wizardKeygenCleanup);
-  }
-  else
-    wizardKeygenCleanup();
-
-}
-
-function wizardKeygenCleanup() {
-  DEBUG_LOG("enigmailSetupWizard.js: wizardKeygenCleanup\n");
-  enigmailKeygenCloseRequest();
-  var enigmailSvc = enigGetSvc();
-  enigmailSvc.invalidateUserIdList();
-
-  var wizard = document.getElementById("enigmailSetupWizard");
-  wizard.goTo("pgComplete");
-}
-
-
 function disableIdSel(doDisable) {
   var idSelectionBox = document.getElementById("idSelection");
   var node = idSelectionBox.firstChild;
@@ -781,134 +1163,34 @@ function checkIdSelection() {
   disableNext(countSelectedId() < 1);
 }
 
-function showPrefDetails() {
-
-  window.openDialog("chrome://enigmail/content/enigmailWizardPrefs.xul",
-            "", "chrome,modal,centerscreen", gEnigModifySettings);
-  return true;
-}
-
 
 function loadLastPage() {
-  var wizard = document.getElementById("enigmailSetupWizard");
+  var wizard = getWizard();
   wizard.canRewind=false;
   wizard.getButton("cancel").disabled = true;
 }
 
 
 function setNewKey() {
-  setNextPage('pgKeyCreate');
   disableNext(false);
   gCreateNewKey = true;
-  document.getElementById("uidSelection").boxObject.element.setAttribute("disabled", "true")
+  document.getElementById("uidSelection").boxObject.element.setAttribute("disabled", "true");
 }
 
 function setUseKey() {
-  setNextPage('pgSummary');
   gCreateNewKey = false;
   document.getElementById("uidSelection").boxObject.element.removeAttribute("disabled");
   onKeySelected();
 }
 
 function setImportKeys() {
-  setNextPage('pgKeyImport');
   gCreateNewKey = false;
   disableNext(false);
   document.getElementById("uidSelection").boxObject.element.setAttribute("disabled", "true");
 }
 
-function displayActions() {
-
-  var currItem=0;
-  function appendDesc(what) {
-    ++currItem;
-    var item = document.getElementById("applyDesc"+currItem);
-    item.value="\u2013 "+what;
-    item.removeAttribute("collapsed");
-  }
-
-  var createKey1=document.getElementById("createPgpKey");
-  var createKey2=document.getElementById("newPgpKey");
-
-  if (gCreateNewKey ||
-      document.getElementById("pgSettings").next == "pgKeyCreate") {
-    setNextPage('pgKeygen');
-    appendDesc(EnigGetString("setupWizard.createKey"));
-  }
-  else {
-    setNextPage('pgComplete');
-    appendDesc(EnigGetString("setupWizard.useKey", gGeneratedKey));
-  }
-
-  var descList=document.getElementById("appliedSettings");
-
-  if (countIdentities() >1) {
-    if (document.getElementById("activateId").value == "1") {
-      appendDesc(EnigGetString("setupWizard.applyAllId"));
-    }
-    else {
-      var idList = "";
-      var node = document.getElementById("idSelection").firstChild;
-      while (node) {
-        if (node.checked) {
-          var identity = gEnigAccountMgr.getIdentity(node.getAttribute("account-id"));
-          idList+="<"+identity.email+"> "
-        }
-        node = node.nextSibling;
-      }
-      appendDesc(EnigGetString("setupWizard.applySomeId", idList));
-    }
-  }
-  else {
-    appendDesc(EnigGetString("setupWizard.applySingleId", idList));
-  }
-
-  if (document.getElementById("signMsg").value== "1") {
-    appendDesc(EnigGetString("setupWizard.signAll"));
-  }
-  else {
-    appendDesc(EnigGetString("setupWizard.signNone"));
-  }
-
-  if (document.getElementById("encryptMsg").value== "1") {
-    appendDesc(EnigGetString("setupWizard.encryptAll"));
-  }
-  else {
-    appendDesc(EnigGetString("setupWizard.encryptNone"));
-  }
-
-  if (document.getElementById("changeSettings").value == "1") {
-    if (gEnigModifySettings["imapOnDemand"] &&
-        gEnigModifySettings["flowedText"] &&
-        gEnigModifySettings["quotedPrintable"] &&
-        gEnigModifySettings["viewPlainText"] &&
-        gEnigModifySettings["composeHTML"]) {
-      appendDesc(EnigGetString("setupWizard.setAllPrefs"));
-    }
-    else if (gEnigModifySettings["imapOnDemand"] ||
-        gEnigModifySettings["flowedText"] ||
-        gEnigModifySettings["quotedPrintable"] ||
-        gEnigModifySettings["viewPlainText"] ||
-        gEnigModifySettings["composeHTML"]) {
-      appendDesc(EnigGetString("setupWizard.setSomePrefs"));
-    }
-    else {
-      appendDesc(EnigGetString("setupWizard.setNoPrefs"));
-    }
-  }
-  else {
-    appendDesc(EnigGetString("setupWizard.setNoPrefs"));
-  }
-}
 
 // Helper function
 function getServersForIdentity(accMgr, identity) {
-  try {
-    // Gecko >= 20
-    return accMgr.getServersForIdentity(identity);
-  }
-  catch(ex) {
-    // Gecko < 20
-    return accMgr.GetServersForIdentity(identity);
-  }
+  return accMgr.getServersForIdentity(identity);
 }
